@@ -51,6 +51,12 @@ private Q_SLOTS:
         QVERIFY(station.coordinate.isValid());
         QCOMPARE(station.timezone.toQTimeZone(), QTimeZone("Europe/Zurich"));
         QCOMPARE(station.country, CountryId{"CH"});
+
+        // Aachen West, very close to the NL border, should be in DE timezone
+        station = KnowledgeDb::stationForIbnr(IBNR{8000404});
+        QVERIFY(station.coordinate.isValid());
+        QCOMPARE(station.timezone.toQTimeZone(), QTimeZone("Europe/Berlin"));
+        QCOMPARE(station.country, CountryId{"DE"});
     }
 
     void testGaresConnexionsIdLookup()
@@ -138,6 +144,15 @@ private Q_SLOTS:
         QCOMPARE(KnowledgeDb::incompatiblePowerPlugs(plugs, sockets), failPlugs);
         QCOMPARE(KnowledgeDb::incompatiblePowerSockets(plugs, sockets), failSockets);
     }
+
+    void testTimezoneForCountry()
+    {
+        using namespace KnowledgeDb;
+
+        QCOMPARE(KnowledgeDb::timezoneForCountry(CountryId{"DE"}).toQTimeZone(), QTimeZone("Europe/Berlin"));
+        QCOMPARE(KnowledgeDb::timezoneForCountry(CountryId{"FR"}).toQTimeZone(), QTimeZone("Europe/Paris"));
+        QCOMPARE(KnowledgeDb::timezoneForCountry(CountryId{"BR"}).toQTimeZone(), QTimeZone());
+    }
 };
 
 QTEST_APPLESS_MAIN(KnowledgeDbTest)
