@@ -20,6 +20,8 @@
 
 #include "kitinerary_export.h"
 #include "datatypes.h"
+#include "organization.h"
+#include "place.h"
 
 class QUrl;
 
@@ -27,8 +29,6 @@ namespace KItinerary {
 
 class ReservationPrivate;
 
-class Organization;
-
 /** Abstract base class for reservations.
  *  @see https://schema.org/Reservation
  */
@@ -42,6 +42,7 @@ class KITINERARY_EXPORT Reservation
     KITINERARY_PROPERTY(KItinerary::Organization, provider, setProvider)
     KITINERARY_PROPERTY(QUrl, url, setUrl)
     KITINERARY_PROPERTY(QVariantList, potentialAction, setPotentialAction)
+    KITINERARY_PROPERTY(QDateTime, modifiedTime, setModifiedTime)
 
     // KDE extensions
     /** Pass type identifier of an associated Apple Wallet boarding pass.
@@ -116,6 +117,38 @@ class KITINERARY_EXPORT FoodEstablishmentReservation : public Reservation
     KITINERARY_PROPERTY(QDateTime, startTime, setStartTime)
 };
 
+/** An event reservation.
+ *  @see https://schema.org/EventReservation
+ *  @see https://developers.google.com/gmail/markup/reference/event-reservation
+ */
+class KITINERARY_EXPORT EventReservation : public Reservation
+{
+    KITINERARY_GADGET(EventReservation)
+};
+
+/** A Rental Car reservation.
+ *  @see https://developers.google.com/gmail/markup/reference/rental-car
+ */
+class KITINERARY_EXPORT RentalCarReservation : public Reservation
+{
+    KITINERARY_GADGET(RentalCarReservation)
+    KITINERARY_PROPERTY(QDateTime, dropoffTime, setDropoffTime)
+    KITINERARY_PROPERTY(QDateTime, pickupTime, setPickupTime)
+    KITINERARY_PROPERTY(KItinerary::Place, pickupLocation, setPickupLocation)
+    KITINERARY_PROPERTY(KItinerary::Place, dropoffLocation, setDropoffLocation)
+};
+
+/** A Taxi reservation.
+ *  @see https://schema.org/TaxiReservation
+ */
+class KITINERARY_EXPORT TaxiReservation : public Reservation
+{
+    KITINERARY_GADGET(TaxiReservation)
+    KITINERARY_PROPERTY(QDateTime, pickupTime, setPickupTime)
+    KITINERARY_PROPERTY(KItinerary::Place, pickupLocation, setPickupLocation)
+};
+
+
 }
 
 Q_DECLARE_METATYPE(KItinerary::FlightReservation)
@@ -123,6 +156,8 @@ Q_DECLARE_METATYPE(KItinerary::LodgingReservation)
 Q_DECLARE_METATYPE(KItinerary::TrainReservation)
 Q_DECLARE_METATYPE(KItinerary::BusReservation)
 Q_DECLARE_METATYPE(KItinerary::FoodEstablishmentReservation)
+Q_DECLARE_METATYPE(KItinerary::EventReservation)
+Q_DECLARE_METATYPE(KItinerary::RentalCarReservation)
 
 #endif // KITINERARY_RESERVATION_H
 
