@@ -35,7 +35,8 @@ namespace CodeGen
     void writeLicenseHeader(QIODevice *out);
     void writeCoordinate(QIODevice *out, const KnowledgeDb::Coordinate &coord);
     void writeCountryIsoCode(QIODevice *out, const QString &isoCode);
-    void writeTimezone(QIODevice *out, Timezones *tzDb, const QByteArray &tzName);
+    void writeTimezone(QIODevice *out, const QByteArray &tzName);
+    void writeTimezoneEnum(QIODevice *out, const QByteArray &tzName);
 }
 
 }
