@@ -16,6 +16,7 @@
 */
 
 #include "timezonedb.h"
+#include "timezonedb_p.h"
 #include "timezonedb_data.cpp"
 
 #include <QTimeZone>
@@ -29,3 +30,13 @@ QTimeZone Timezone::toQTimeZone() const
     }
     return QTimeZone(timezone_names + offset);
 }
+
+Timezone KItinerary::KnowledgeDb::timezoneForCountry(CountryId country)
+{
+    const auto it = std::lower_bound(std::begin(country_timezone_map), std::end(country_timezone_map), country);
+    if (it != std::end(country_timezone_map) && (*it).country == country) {
+        return (*it).timezone;
+    }
+
+    return {};
+}
