@@ -27,14 +27,16 @@ class QTimeZone;
 
 namespace KItinerary {
 namespace KnowledgeDb {
+    class CountryId;
+    enum class Tz : uint16_t;
 
     /** Timezone type as used in the database.
      *  @note Do not use in API/ABI.
      */
     struct Timezone {
         inline constexpr Timezone() = default;
-        inline explicit constexpr Timezone(uint16_t o)
-            : offset(o)
+        inline constexpr Timezone(Tz tz)
+            : offset(static_cast<uint16_t>(tz))
         {}
 
         /** Returns the corresponding QTimeZone. */
@@ -43,6 +45,11 @@ namespace KnowledgeDb {
         // offset into timezone name string table
         uint16_t offset = std::numeric_limits<uint16_t>::max();
     };
+
+    /** Returns the timezone for the given country, as long as there is exactly
+     *  one timezone used in that country.
+     */
+    KITINERARY_EXPORT Timezone timezoneForCountry(CountryId country);
 }
 }
 
