@@ -30,6 +30,7 @@
 #endif
 
 #include <cmath>
+#include <unordered_map>
 
 using namespace KItinerary;
 
@@ -37,22 +38,28 @@ namespace KItinerary {
 class PdfImagePrivate : public QSharedData {
 public:
 #ifdef HAVE_POPPLER
-    void load(Stream *str, int width, int height, GfxImageColorMap *colorMap);
+    QImage load(Stream *str, GfxImageColorMap *colorMap);
+
+    std::unique_ptr<GfxImageColorMap> m_colorMap;
 #endif
 
-    int m_ref = -1;
+    int m_refNum = -1;
+    int m_refGen = -1;
     PdfPagePrivate *m_page = nullptr;
-    QImage m_img;
-    double m_x = NAN;
-    double m_y = NAN;
+    QTransform m_transform;
     int m_width = 0;
     int m_height = 0;
+    int m_sourceWidth = 0;
+    int m_sourceHeight = 0;
     QImage::Format m_format = QImage::Format_Invalid;
 };
 
 class PdfPagePrivate : public QSharedData {
 public:
+    void load();
+
     int m_pageNum = -1;
+    bool m_loaded = false;
     QString m_text;
     std::vector<PdfImage> m_images;
     PdfDocumentPrivate *m_doc;
@@ -60,8 +67,13 @@ public:
 
 class PdfDocumentPrivate {
 public:
-    QByteArray m_pdfData; // needs to be kept alive as long as the Poppler::PdfDoc instance lives
-    std::vector<PdfImage> m_images;
+    // needs to be kept alive as long as the Poppler::PdfDoc instance lives
+    QByteArray m_pdfData;
+    // this contains the actually loaded/decoded image data
+    // and is referenced by the object id from PdfImage to avoid
+    // expensive loading/decoding of multiple occurrences of the same image
+    // image data in here is stored in its source form, without applied transformations
+    std::unordered_map<int, QImage> m_imageData;
     std::vector<PdfPage> m_pages;
 #ifdef HAVE_POPPLER
     std::unique_ptr<PDFDoc> m_popplerDoc;
@@ -81,16 +93,17 @@ static GlobalParams* popplerGlobalParams()
 class ExtractorOutputDevice : public TextOutputDev
 {
 public:
-    ExtractorOutputDevice(PdfDocumentPrivate *dd);
+    ExtractorOutputDevice();
     bool needNonText() override { return true; }
     void drawImage(GfxState *state, Object *ref, Stream *str, int width, int height, GfxImageColorMap *colorMap, bool interpolate, int *maskColors, bool inlineImg) override;
 
     std::vector<PdfImage> m_images;
 
 private:
-    PdfDocumentPrivate *d;
+    QTransform m_transform;
 };
 
+#ifndef HAVE_POPPLER_0_69
 class ImageLoaderOutputDevice : public OutputDev
 {
 public:
@@ -102,29 +115,32 @@ public:
     bool useDrawChar() override { return false; }
 
     void drawImage(GfxState *state, Object *ref, Stream *str, int width, int height, GfxImageColorMap *colorMap, bool interpolate, int *maskColors, bool inlineImg) override;
+    QImage image() const { return m_image; }
 
 private:
     PdfImagePrivate *d;
+    QImage m_image;
 };
 #endif
+#endif
 
 }
 
 #ifdef HAVE_POPPLER
-void PdfImagePrivate::load(Stream* str, int width, int height, GfxImageColorMap* colorMap)
+QImage PdfImagePrivate::load(Stream* str, GfxImageColorMap* colorMap)
 {
-    m_img = QImage(width, height, m_format);
+    auto img = QImage(m_sourceWidth, m_sourceHeight, m_format);
     const auto bytesPerPixel = colorMap->getNumPixelComps();
-    std::unique_ptr<ImageStream> imgStream(new ImageStream(str, width, bytesPerPixel, colorMap->getBits()));
+    std::unique_ptr<ImageStream> imgStream(new ImageStream(str, m_sourceWidth, bytesPerPixel, colorMap->getBits()));
     imgStream->reset();
 
     switch (m_format) {
         case QImage::Format_RGB888:
-            for (int i = 0; i < height; ++i) {
+            for (int i = 0; i < m_sourceHeight; ++i) {
                 const auto row = imgStream->getLine();
-                auto imgData = m_img.scanLine(i);
+                auto imgData = img.scanLine(i);
                 GfxRGB rgb;
-                for (int j = 0; j < width; ++j) {
+                for (int j = 0; j < m_sourceWidth; ++j) {
                     colorMap->getRGB(row + (j * bytesPerPixel), &rgb);
                     *imgData++ = colToByte(rgb.r);
                     *imgData++ = colToByte(rgb.g);
@@ -133,11 +149,11 @@ void PdfImagePrivate::load(Stream* str, int width, int height, GfxImageColorMap*
             }
             break;
         case QImage::Format_Grayscale8:
-            for (int i = 0; i < height; ++i) {
+            for (int i = 0; i < m_sourceHeight; ++i) {
                 const auto row = imgStream->getLine();
-                auto imgData = m_img.scanLine(i);
+                auto imgData = img.scanLine(i);
                 GfxGray gray;
-                for (int j = 0; j < width; ++j) {
+                for (int j = 0; j < m_sourceWidth; ++j) {
                     colorMap->getGray(row + j, &gray);
                     *imgData++ = colToByte(gray);
                 }
@@ -148,15 +164,13 @@ void PdfImagePrivate::load(Stream* str, int width, int height, GfxImageColorMap*
     }
     imgStream->close();
 
-    if (m_width != width || m_height != height) {
-        m_img = m_img.scaled(m_width, m_height);
-    }
+    m_page->m_doc->m_imageData[m_refNum] = img;
+    return img;
 }
 
 
-ExtractorOutputDevice::ExtractorOutputDevice(PdfDocumentPrivate *dd)
+ExtractorOutputDevice::ExtractorOutputDevice()
     : TextOutputDev(nullptr, false, 0, false, false)
-    , d(dd)
 {
 }
 
@@ -167,21 +181,10 @@ void ExtractorOutputDevice::drawImage(GfxState* state, Object* ref, Stream* str,
     Q_UNUSED(maskColors);
     Q_UNUSED(inlineImg);
 
-    if (!colorMap || !colorMap->isOk()) {
+    if (!colorMap || !colorMap->isOk() || !ref || !ref->isRef()) {
         return;
     }
 
-    // check for duplicate occurances of ref->getRef().num
-    if (ref->isRef()) {
-        auto it = std::find_if(d->m_images.begin(), d->m_images.end(), [ref](const PdfImage &other) {
-            return other.d->m_ref == ref->getRef().num;
-        });
-        if (it != d->m_images.end()) {
-            m_images.push_back(*it);
-            return;
-        }
-    }
-
     QImage::Format format;
     if (colorMap->getColorSpace()->getMode() == csIndexed) {
         format = QImage::Format_RGB888;
@@ -194,10 +197,14 @@ void ExtractorOutputDevice::drawImage(GfxState* state, Object* ref, Stream* str,
     }
 
     PdfImage pdfImg;
-    if (ref->isRef()) {
-        pdfImg.d->m_ref = ref->getRef().num;
-    }
+    pdfImg.d->m_refNum = ref->getRef().num;
+    pdfImg.d->m_refGen = ref->getRef().gen;
 
+#ifdef HAVE_POPPLER_0_69
+    pdfImg.d->m_colorMap.reset(colorMap->copy());
+#endif
+    pdfImg.d->m_sourceHeight = height;
+    pdfImg.d->m_sourceWidth = width;
     pdfImg.d->m_width = width;
     pdfImg.d->m_height = height;
     // deal with aspect-ratio changing scaling
@@ -205,17 +212,18 @@ void ExtractorOutputDevice::drawImage(GfxState* state, Object* ref, Stream* str,
     const auto targetAspectRatio = state->getCTM()[0] / -state->getCTM()[3];
     if (!qFuzzyCompare(sourceAspectRatio, targetAspectRatio) && qFuzzyIsNull(state->getCTM()[1]) && qFuzzyIsNull(state->getCTM()[2])) {
         if (targetAspectRatio > sourceAspectRatio) {
-            pdfImg.d->m_width = height * state->getCTM()[0] / -state->getCTM()[3];
+            pdfImg.d->m_width = width * targetAspectRatio / sourceAspectRatio;
         } else {
-            pdfImg.d->m_height = width * -state->getCTM()[3] / state->getCTM()[0];
+            pdfImg.d->m_height = height * sourceAspectRatio / targetAspectRatio;
         }
     }
-    pdfImg.d->m_x = state->getCTM()[4];
-    pdfImg.d->m_y = state->getCTM()[5];
+    const auto ctm = state->getCTM();
+    pdfImg.d->m_transform = QTransform(ctm[0], ctm[1], ctm[2], ctm[3], ctm[4], ctm[5]);
     pdfImg.d->m_format = format;
     m_images.push_back(pdfImg);
 }
 
+#ifndef HAVE_POPPLER_0_69
 ImageLoaderOutputDevice::ImageLoaderOutputDevice(PdfImagePrivate* dd)
     : d(dd)
 {
@@ -224,21 +232,24 @@ ImageLoaderOutputDevice::ImageLoaderOutputDevice(PdfImagePrivate* dd)
 void ImageLoaderOutputDevice::drawImage(GfxState *state, Object *ref, Stream *str, int width, int height, GfxImageColorMap *colorMap, bool interpolate, int *maskColors, bool inlineImg)
 {
     Q_UNUSED(state);
+    Q_UNUSED(height);
+    Q_UNUSED(width);
     Q_UNUSED(interpolate);
     Q_UNUSED(maskColors);
     Q_UNUSED(inlineImg);
 
-    if (!colorMap || !colorMap->isOk()) {
+    if (!colorMap || !colorMap->isOk() || !ref) {
         return;
     }
 
-    if (ref->isRef() && d->m_ref != ref->getRef().num) {
+    if (ref->isRef() && d->m_refNum != ref->getRef().num) {
         return;
     }
 
-    d->load(str, width, height, colorMap);
+    m_image = d->load(str, colorMap);
 }
 #endif
+#endif
 
 PdfImage::PdfImage()
     : d(new PdfImagePrivate)
@@ -259,21 +270,82 @@ int PdfImage::width() const
     return d->m_width;
 }
 
-QImage PdfImage::image() const
+int PdfImage::sourceHeight() const
 {
-    if (!d->m_img.isNull() || d->m_ref < 0) {
-        return d->m_img;
+    return d->m_sourceHeight;
+}
+
+int PdfImage::sourceWidth() const
+{
+    return d->m_sourceWidth;
+}
+
+QTransform PdfImage::transform() const
+{
+    return d->m_transform;
+}
+
+QImage PdfImage::sourceImage() const
+{
+    const auto it = d->m_page->m_doc->m_imageData.find(d->m_refNum);
+    if (it != d->m_page->m_doc->m_imageData.end()) {
+        return (*it).second;
     }
 
 #ifdef HAVE_POPPLER
     QScopedValueRollback<GlobalParams*> globalParamResetter(globalParams, popplerGlobalParams());
 
+#ifdef HAVE_POPPLER_0_69
+    const auto xref = d->m_page->m_doc->m_popplerDoc->getXRef();
+    const auto obj = xref->fetch(d->m_refNum, d->m_refGen);
+    return d->load(obj.getStream(), d->m_colorMap.get());
+#else
     std::unique_ptr<ImageLoaderOutputDevice> device(new ImageLoaderOutputDevice(d.data()));
     d->m_page->m_doc->m_popplerDoc->displayPageSlice(device.get(), d->m_page->m_pageNum + 1, 72, 72, 0, false, true, false, -1, -1, -1, -1);
+    return device->image();
+#endif
+
+#else
+    return {};
 #endif
-    return d->m_img;
 }
 
+QImage PdfImage::image() const
+{
+    const auto img = sourceImage();
+    if (d->m_width != d->m_sourceWidth || d->m_height != d->m_sourceHeight) {
+        return img.scaled(d->m_width, d->m_height);
+    }
+    return img;
+}
+
+int PdfImage::objectId() const
+{
+    return d->m_refNum;
+}
+
+
+void PdfPagePrivate::load()
+{
+    if (m_loaded) {
+        return;
+    }
+
+#ifdef HAVE_POPPLER
+    QScopedValueRollback<GlobalParams*> globalParamResetter(globalParams, popplerGlobalParams());
+    ExtractorOutputDevice device;
+    m_doc->m_popplerDoc->displayPageSlice(&device, m_pageNum + 1, 72, 72, 0, false, true, false, -1, -1, -1, -1);
+    const auto pageRect = m_doc->m_popplerDoc->getPage(m_pageNum + 1)->getCropBox();
+    std::unique_ptr<GooString> s(device.getText(pageRect->x1, pageRect->y1, pageRect->x2, pageRect->y2));
+
+    m_text = QString::fromUtf8(s->getCString());
+    m_images = std::move(device.m_images);
+    for (auto it = m_images.begin(); it != m_images.end(); ++it) {
+        (*it).d->m_page = this;
+    }
+#endif
+    m_loaded = true;
+}
 
 PdfPage::PdfPage()
     : d(new PdfPagePrivate)
@@ -286,6 +358,7 @@ PdfPage& PdfPage::operator=(const PdfPage&) = default;
 
 QString PdfPage::text() const
 {
+    d->load();
     return d->m_text;
 }
 
@@ -301,11 +374,11 @@ QString PdfPage::textInRect(double left, double top, double right, double bottom
 #ifdef HAVE_POPPLER
     QScopedValueRollback<GlobalParams*> globalParamResetter(globalParams, popplerGlobalParams());
 
-    std::unique_ptr<ExtractorOutputDevice> device(new ExtractorOutputDevice(d->m_doc));
-    d->m_doc->m_popplerDoc->displayPageSlice(device.get(), d->m_pageNum + 1, 72, 72, 0, false, true, false, -1, -1, -1, -1);
+    ExtractorOutputDevice device;
+    d->m_doc->m_popplerDoc->displayPageSlice(&device, d->m_pageNum + 1, 72, 72, 0, false, true, false, -1, -1, -1, -1);
     const auto pageRect = d->m_doc->m_popplerDoc->getPage(d->m_pageNum + 1)->getCropBox();
-    std::unique_ptr<GooString> s(device->getText(ratio(pageRect->x1, pageRect->x2, left), ratio(pageRect->y1, pageRect->y2, top),
-                                                 ratio(pageRect->x1, pageRect->x2, right), ratio(pageRect->y1, pageRect->y2, bottom)));
+    std::unique_ptr<GooString> s(device.getText(ratio(pageRect->x1, pageRect->x2, left), ratio(pageRect->y1, pageRect->y2, top),
+                                                ratio(pageRect->x1, pageRect->x2, right), ratio(pageRect->y1, pageRect->y2, bottom)));
     return QString::fromUtf8(s->getCString());
 #else
     Q_UNUSED(left);
@@ -318,16 +391,19 @@ QString PdfPage::textInRect(double left, double top, double right, double bottom
 
 int PdfPage::imageCount() const
 {
+    d->load();
     return d->m_images.size();
 }
 
 PdfImage PdfPage::image(int index) const
 {
+    d->load();
     return d->m_images[index];
 }
 
 QVariantList PdfPage::imagesVariant() const
 {
+    d->load();
     QVariantList l;
     l.reserve(imageCount());
     std::for_each(d->m_images.begin(), d->m_images.end(), [&l](const PdfImage& img) { l.push_back(QVariant::fromValue(img)); });
@@ -336,14 +412,15 @@ QVariantList PdfPage::imagesVariant() const
 
 QVariantList PdfPage::imagesInRect(double left, double top, double right, double bottom) const
 {
+    d->load();
     QVariantList l;
 #ifdef HAVE_POPPLER
     QScopedValueRollback<GlobalParams*> globalParamResetter(globalParams, popplerGlobalParams());
     const auto pageRect = d->m_doc->m_popplerDoc->getPage(d->m_pageNum + 1)->getCropBox();
 
     for (const auto &img : d->m_images) {
-        if ((img.d->m_x >= ratio(pageRect->x1, pageRect->x2, left) && img.d->m_x <= ratio(pageRect->x1, pageRect->x2, right)) &&
-            (img.d->m_y >= ratio(pageRect->y1, pageRect->y2, top)  && img.d->m_y <= ratio(pageRect->y1, pageRect->y2, bottom)))
+        if ((img.d->m_transform.dx() >= ratio(pageRect->x1, pageRect->x2, left) && img.d->m_transform.dx() <= ratio(pageRect->x1, pageRect->x2, right)) &&
+            (img.d->m_transform.dy() >= ratio(pageRect->y1, pageRect->y2, top)  && img.d->m_transform.dy() <= ratio(pageRect->y1, pageRect->y2, bottom)))
         {
             l.push_back(QVariant::fromValue(img));
         }
@@ -373,27 +450,13 @@ QString PdfDocument::text() const
     return text;
 }
 
-int PdfDocument::imageCount() const
-{
-    return d->m_images.size();
-}
-
-PdfImage PdfDocument::image(int index) const
-{
-    return d->m_images[index];
-}
-
-QVariantList PdfDocument::imagesVariant() const
-{
-    QVariantList l;
-    l.reserve(imageCount());
-    std::for_each(d->m_images.begin(), d->m_images.end(), [&l](const PdfImage& img) { l.push_back(QVariant::fromValue(img)); });
-    return l;
-}
-
 int PdfDocument::pageCount() const
 {
-    return d->m_pages.size();
+#ifdef HAVE_POPPLER
+    return d->m_popplerDoc->getNumPages();
+#else
+    return 0;
+#endif
 }
 
 PdfPage PdfDocument::page(int index) const
@@ -401,6 +464,11 @@ PdfPage PdfDocument::page(int index) const
     return d->m_pages[index];
 }
 
+int PdfDocument::fileSize() const
+{
+    return d->m_pdfData.size();
+}
+
 QVariantList PdfDocument::pagesVariant() const
 {
     QVariantList l;
@@ -429,22 +497,11 @@ PdfDocument* PdfDocument::fromData(const QByteArray &data, QObject *parent)
         return nullptr;
     }
 
-    std::unique_ptr<ExtractorOutputDevice> device(new ExtractorOutputDevice(doc->d.get()));
     doc->d->m_pages.reserve(popplerDoc->getNumPages());
     for (int i = 0; i < popplerDoc->getNumPages(); ++i) {
-        popplerDoc->displayPageSlice(device.get(), i + 1, 72, 72, 0, false, true, false, -1, -1, -1, -1);
-        const auto pageRect = popplerDoc->getPage(i + 1)->getCropBox();
-        std::unique_ptr<GooString> s(device->getText(pageRect->x1, pageRect->y1, pageRect->x2, pageRect->y2));
-        std::copy(device->m_images.begin(), device->m_images.end(), std::back_inserter(doc->d->m_images));
-
         PdfPage page;
         page.d->m_pageNum = i;
         page.d->m_doc = doc->d.get();
-        page.d->m_text = QString::fromUtf8(s->getCString());
-        page.d->m_images = std::move(device->m_images);
-        for (auto it = page.d->m_images.begin(); it != page.d->m_images.end(); ++it) {
-            (*it).d->m_page = page.d.data();
-        }
         doc->d->m_pages.push_back(page);
     }
 
